use std::convert::{TryFrom, TryInto};
use serde::{Deserialize, Serialize};
use trust_dns_proto::error::ProtoError;

use crate::dns;
use super::name::SerdeName;
use super::class::DNSClass;
use super::rdata::RData;


#[derive(Deserialize, Serialize, Clone)]
pub struct Record {
    #[serde(rename = "Name")]
    pub name: SerdeName,
    // TODO: Make class optional, default to IN
    #[serde(rename = "Class")]
    pub dns_class: DNSClass,
    #[serde(rename = "TTL")]
    pub ttl: u32,
    #[serde(flatten)]
    pub rdata: RData,
}

impl From<dns::Record> for Record {
    fn from(record: dns::Record) -> Record {
        Record {
            name: SerdeName(record.name().clone()),
            dns_class: record.dns_class().into(),
            ttl: record.ttl(),
            rdata: record.into_data().into(),
        }
    }
}

impl TryFrom<Record> for dns::Record {
    type Error = ProtoError;

    fn try_from(record: Record) -> Result<Self, Self::Error> {
        let mut trust_dns_record = dns::Record::from_rdata(record.name.into_inner(), record.ttl, record.rdata.try_into()?);
        trust_dns_record.set_dns_class(record.dns_class.into());
        Ok(trust_dns_record)
    }
}


pub type RecordList = Vec<Record>;

pub struct RecordListParseError {
    pub bad_records: Vec<Record>,
    pub zone: dns::Name,
}

pub trait ParseRecordList {
    fn try_into_dns_type(self, zone: dns::Name) -> Result<Vec<dns::Record>, RecordListParseError>;
}

impl ParseRecordList for RecordList {
    fn try_into_dns_type(self, zone: dns::Name) -> Result<Vec<dns::Record>, RecordListParseError> {
        // TODO: What about relative names (also in cnames and stuff)
        let mut bad_records = Vec::new();
        let mut records: Vec<dns::Record> = Vec::new();

        for record in self.into_iter() {
            let this_record = record.clone();
            if let Ok(record) = record.try_into() {
                records.push(record);
            } else {
                bad_records.push(this_record.clone());
            }
        }

        if !bad_records.is_empty() {
            return Err(RecordListParseError {
                zone,
                bad_records,
            });
        }
        return Ok(records)
    }
}